@@ -17,6 +17,8 @@
 
 (function() {
 
+    var my_cms = {};
+    var error_marker = null;
     $.ajaxSetup({
         "contentType": "application/json",
     })
@@ -30,6 +32,17 @@
         console.log(event)
         console.log('ajaxSettings:')
         console.log(ajaxSettings)
+        
+        if(jqXHR.status == 400 && (ajaxSettings['url'] == '/add_index' || ajaxSettings['url'] == '/edit_index')) {
+            var ls = $.parseJSON(jqXHR.responseText)['reason']
+            ls = RegExp('\\(in line: ([0-9]+)\\)').exec(ls)
+            if (error_marker) {
+                error_marker.clean()
+            }
+            if (ls){
+                error_marker = my_cms['index_code'].markText({line : ls[1]-1,ch : 0},{line : ls[1]-1,ch : 99999},"cm-error")
+            }
+        }
     })
 
     $('#indexDetailsNavigation').live('click', function(){
@@ -37,67 +50,71 @@
     });
 
     ko.bindingHandlers.codeMirror = function() {
-        var my_cm = null;
         return {
             init: function(element, valueAccessor, allBindingsAccessor, context) {
-                var options = allBindingsAccessor().codeMirrorOptions || {}
-                var modelValue = valueAccessor()
-                var value = ko.utils.unwrapObservable(valueAccessor())
-                var el = $(element)
+              var options = allBindingsAccessor().codeMirrorOptions || {}
+              var modelValue = valueAccessor()
+              var value = ko.utils.unwrapObservable(valueAccessor())
+              var el = $(element)
 
-                setTimeout(function() {
-                    options.onChange = function(from, to, text, next) {
-                        if (ko.isWriteableObservable(modelValue)) {
-                            modelValue(my_cm.getValue())
-                        }
-                    }
-                    my_cm = CodeMirror.fromTextArea(element, options)
-                    if(value) {
-                        my_cm.setValue(value)
-                    }
-                }, 0)
+              setTimeout(function() {
+                      options.onChange = function(from, to, text, next) {
+                      if (ko.isWriteableObservable(modelValue)) {
+                      modelValue(my_cms[el.attr('id')].getValue())
+                      }
+                      }
+                      my_cms[el.attr('id')] = CodeMirror.fromTextArea(element, options)
 
-                ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
-                    setTimeout(function(){
-                        my_cm.toTextArea()
-                    }, 0)
-                });
+                      if(value) {
+                      my_cms[el.attr('id')].setValue(value)
+                      }
+                      }, 0)
 
-            }
-            , update: function(element, valueAccessor) {
-                var el = $(element)
-                var value = ko.utils.unwrapObservable(valueAccessor())
-                var id = el.attr('id')
-                var update = function(val) {
-                    var update_tries = 0
-                    function later() {
-                        setTimeout(function() {
-                            updater()
-                        }, 5)
-                    }
-                    function updater() {
-                        if(!my_cm) {
-                            update_tries++
-                            if (update_tries < 5) {
-                                later()
-                            } else {
-                                console.log("Failed to activate codemirror")
-                            }
-                        } else {
-                            if (val) {
-                                if(val != my_cm.getValue()) {
-                                    my_cm.setValue(val)
-                                }
-                            }
-                        }
-                    }
-                    return updater
-                }
-                if (id !== undefined && id !== '') {
-                    var upd = update(value)
-                    upd()
-                }
-            }
+          ko.utils.domNodeDisposal.addDisposeCallback(element, function () {
+                  setTimeout(function(){
+                      my_cms[el.attr('id')].toTextArea()
+                      }, 0)
+                  });
+
+      }
+      , update: function(element, valueAccessor) {
+          var el = $(element)
+              var value = ko.utils.unwrapObservable(valueAccessor())
+              var id = el.attr('id')
+              var update = function(val) {
+                  var update_tries = 0
+                      function later() {
+                          setTimeout(function() {
+                                  updater()
+                                  }, 5)
+                      }
+                  function updater() {
+                      if(!my_cms[el.attr('id')]) {
+                          update_tries++
+                              if (update_tries < 5) {
+                                  later()
+                              } else {
+                                  console.log("Failed to activate codemirror")
+                              }
+                      } else {
+                          if (val != null) {
+                              if(val != my_cms[el.attr('id')].getValue()) {
+                                  my_cms[el.attr('id')].setValue(val)
+                              }
+                          }
+                          if (error_marker) {
+                              error_marker.clear()
+                              error_marker = null
+                          }
+                      }
+                  }
+                  return updater
+              }
+          if (id !== undefined && id !== '') {
+              var upd = update(value)
+                  upd()
+          }
+      }
         }
     }()
 
@@ -213,6 +230,12 @@
         self.show_code = ko.observable(null)
         self.class_names = ko.observableArray([])
         self.class_name = ko.observable('')
+        self.is_normal_code = ko.observable(true)
+         
+        if (name == null) {
+            self.simplified_code = ""
+            self.python_code = ""
+        }
 
         self.validation = new kk.Validation( self, {
             form_name : [   new kk.validators.Required("Enter name"),
@@ -221,39 +244,58 @@
 
         self.open_save_modal = function() {
             // look for classes in index code
-            var classes = self.index_code().match(/^class\s+[a-zA-Z_]\w*[\(|:]/gm)
+            if (self.is_normal_code()) {
+                var classes = self.index_code().match(/^class\s+[a-zA-Z_]\w*[\(|:]/gm)
 
-            if(!classes) {
-                application.add_error("Can't find any index class in your code.")
-                return
+                    if(!classes) {
+                        application.add_error("Can't find any index class in your code.")
+                            return
+                    }
+
+                // ['class Klass(', ['class Klass2:']] => ['Klass', 'Klass2']
+                for (var i = 0; i < classes.length; i++) classes[i] = classes[i].split(' ')[1].slice(0, -1)
+                    classes = ko.utils.arrayGetDistinctValues(classes)
+                        self.class_names(classes)
+                        self.class_name(classes[0])
+
+                $("#save_plain_modal").modal('show')
+            } else {
+                self._create()
             }
-
-            // ['class Klass(', ['class Klass2:']] => ['Klass', 'Klass2']
-            for (var i = 0; i < classes.length; i++) classes[i] = classes[i].split(' ')[1].slice(0, -1)
-            classes = ko.utils.arrayGetDistinctValues(classes)
-            self.class_names(classes)
-            self.class_name(classes[0])
-
-            $("#save_plain_modal").modal('show')
         }
 
 
         self.open_save_edit_modal = function() {
-            // look for classes in index code
-            var classes = self.index_code().match(/^class\s+[a-zA-Z_]\w*[\(|:]/gm)
+            if (self.is_normal_code()) {
+                // look for classes in index code
+                var classes = self.index_code().match(/^class\s+[a-zA-Z_]\w*[\(|:]/gm)
 
-            if(!classes) {
-                application.add_error("Can't find any index class in your code.")
-                return
+                    if(!classes) {
+                        application.add_error("Can't find any index class in your code.")
+                            return
+                    }
+
+                // ['class Klass(', ['class Klass2:']] => ['Klass', 'Klass2']
+                for (var i = 0; i < classes.length; i++) classes[i] = classes[i].split(' ')[1].slice(0, -1)
+                    classes = ko.utils.arrayGetDistinctValues(classes)
+                        self.class_names(classes)
+                        self.class_name(classes[0])
+
+                $("#save_edit_plain_modal").modal('show')
+            } else {
+                self._edit()
             }
+        }
 
-            // ['class Klass(', ['class Klass2:']] => ['Klass', 'Klass2']
-            for (var i = 0; i < classes.length; i++) classes[i] = classes[i].split(' ')[1].slice(0, -1)
-            classes = ko.utils.arrayGetDistinctValues(classes)
-            self.class_names(classes)
-            self.class_name(classes[0])
-
-            $("#save_edit_plain_modal").modal('show')
+        self.switch_new_code = function () {
+            if (self.is_normal_code()) {
+                self.python_code = self.index_code()
+                self.index_code(self.simplified_code)
+            } else {
+                self.simplified_code = self.index_code()
+                self.index_code(self.python_code)
+            }
+            self.is_normal_code(!self.is_normal_code())
         }
 
         self.save = function() {
@@ -280,9 +322,8 @@
 
             var code_header = "# " + self.form_name() + "\n"
             code_header += "# " + self.class_name() + '\n'
-            code_header += header_footer + "\n\n"
+            code_header += header_footer + "\n"
             var code = ''
-
             if(self.index_code().split('\n')[2].indexOf(header_footer) == 0)
                 code = self.index_code().split('\n').splice(4).join('\n')
             else
@@ -295,7 +336,9 @@
         self.insert_plain = function(){
             $.get('/get_index_header')
             .success(function(data) {
+                    /*console.log(data)*/
                 lines = data.split('\n')
+                /*console.log(self.index_code())*/
                 self.index_code(lines.slice(3).join('\n'))
             })
         }
@@ -306,16 +349,30 @@
             }
         })
 
+        self.is_normal_code.subscribe(function(newValue){
+            if(my_cms['index_code']){
+                my_cms['index_code'].setOption('mode',self.is_normal_code() ? 'python' : 'text')
+            }
+        })
+        
         self.get_index_code = function() {
-            $.get('/get_index_code/' + self.name, {},
-                function(data) {
+            $.get('/get_index_code/' + self.name ,{code_switch : JSON.stringify("S")}, function(data) {
+                if(!self.is_normal_code()){
                     self.index_code(data)
-                    if(application.index_changed != null) {
-                        $('button.btn-reindex').popover({"placement": "top", title: application.index_changed}).popover('show')
-                        application.index_changed = null
-                    }
                 }
-            )
+                self.simplified_code = data
+                })
+            $.get('/get_index_code/' + self.name,{code_switch : JSON.stringify("P")}, function(data) {
+                if (self.is_normal_code()){
+                    self.index_code(data)
+                }
+                self.python_code = data
+                })
+
+            if(application.index_changed != null) {
+                $('button.btn-reindex').popover({"placement": "top", title: application.index_changed}).popover('show')
+                application.index_changed = null
+            }
         }
 
         self.get_index_template = function() {
@@ -434,9 +491,28 @@
                 self.show_code(!self.show_code())
                 if(!self.show_code()) {
                     self.get_details()
+                } else { // XXX: fix it
+                    setTimeout(function() {
+                        my_cms['index_code'].setOption('mode',self.is_normal_code() ? 'python' : 'text')}, 0)
                 }
             }
         }
+        
+
+        self.switch_code = function () {
+            if (self.is_normal_code()) {
+                $.get('/get_index_code/' + self.name ,{code_switch : JSON.stringify("S")}, function(data) {
+                    self.simplified_code = data 
+                    self.index_code(data)
+                    })
+            } else {
+                $.get('/get_index_code/' + self.name ,{code_switch : JSON.stringify("P")}, function(data) {
+                    self.python_code = data 
+                    self.index_code(data)
+                    })
+            }
+            self.is_normal_code(!self.is_normal_code())
+        }
 
         self.code_details_text = function() {
             if (!self.show_code()) {
@@ -867,6 +943,7 @@
                 this.before({}, function() {
                     // Dropdown clearMenus() is not called when we using Sammy.js...
                     $('[data-toggle="dropdown"]').parent().removeClass('open')
+                    $('div.popover').remove()
                     self.has_indexes_names()
                     self.clean_iface()
                 })
@@ -931,4 +1008,4 @@
     application = new DBViewModel()
 
     ko.applyBindings(application)
-})()
\ No newline at end of file
+})()
